/*******************************************************************************
 * @file     dma.h
 * @brief    DMA driver header file.
 * @version  V1.6.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/


#ifndef DMA_H
#define DMA_H

#ifdef __cplusplus
extern "C" {
#endif


#include "spc2168.h"




/**
 *  @brief DMA channel definition
 */
typedef enum
{
    DMA_CH0               = 0,                /*!< DMA channel 0    */
    DMA_CH1               = 1,                /*!< DMA channel 1    */
    DMA_CH2               = 2,                /*!< DMA channel 2    */
    DMA_CH3               = 3,                /*!< DMA channel 3    */
    DMA_CH4               = 4,                /*!< DMA channel 4    */
    DMA_CH5               = 5,                /*!< DMA channel 5    */
} DMA_ChannelEnum;




/**
 *  @brief DMA transfer type definition
 */
typedef enum
{
    DMA_MEMORY_TO_MEMORY            = 0,      /*!< Memory to Memory         */
    DMA_MEMORY_TO_PERIPHERAL        = 1,      /*!< Memory to Peripheral     */
    DMA_PERIPHERAL_TO_MEMORY        = 2,      /*!< Peripheral to Memory     */
    DMA_PERIPHERAL_TO_PERIPHERAL    = 3,      /*!< Peripheral to Peripheral */
} DMA_TransferTypeEnum;




/**
 *  @brief DMA burst transaction length definition
 */
typedef enum
{
    DMA_BURST_TRANS_1_WORD          = 0,      /*!< 1 word (a word = 32 bit) */
    DMA_BURST_TRANS_4_WORDS         = 1,      /*!< 4 words                  */
    DMA_BURST_TRANS_8_WORDS         = 3,      /*!< 8 words                  */
    DMA_BURST_TRANS_16_WORDS        = 4,      /*!< 16 words                 */
    DMA_BURST_TRANS_32_WORDS        = 5,      /*!< 32 words                 */
    DMA_BURST_TRANS_64_WORDS        = 6,      /*!< 64 words                 */
    DMA_BURST_TRANS_128_WORDS       = 7,      /*!< 128 words                */
    DMA_BURST_TRANS_256_WORDS       = 8,      /*!< 256 words                */
} DMA_BurstTransEnum;




/**
 *  @brief DMA address mode definition
 */
typedef enum
{
    DMA_ADDRESS_MODE_INCREASE       = 0,      /*!< Address increment   */
    DMA_ADDRESS_MODE_DECREASE       = 1,      /*!< Address decrement   */
    DMA_ADDRESS_NO_CHANGE           = 2,      /*!< Address no-change   */
} DMA_AddressModeEnum;




/**
 *  @brief DMA handshaking type definition
 */
typedef enum
{
    DMA_HANDSHAKE_BY_HARDWARE       = 0,      /*!< Hardware handshaking */
    DMA_HANDSHAKE_BY_SOFTWARE       = 1,      /*!< Software handshaking */
} DMA_HandshakeTypeEnum;




/**
 *  @brief DMA handshaking polarity definition
 */
typedef enum
{
    DMA_HANDSHAKE_POL_ACTIVE_HIGH   = 0,      /*!< Handshaking interface polarity = Active High */
    DMA_HANDSHAKE_POL_ACTIVE_LOW    = 1,      /*!< Handshaking interface polarity = Active Low  */
} DMA_HandshakePolEnum;




/**
 *  @brief DMA source and destination peripheral definition
 */
typedef enum
{
    DMA_SPER_UART_RX                = 0,      /*!< Source peripheral      = UART_RX */
    DMA_SPER_SSP_RX                 = 2,      /*!< Source peripheral      = SSP_RX  */

    DMA_DPER_UART_TX                = 1,      /*!< Destination peripheral = UART_TX */
    DMA_DPER_SSP_TX                 = 3,      /*!< Destination peripheral = SSP_TX  */
} DMA_PeripheralEnum;




/******************************************************************************
 * @brief      Set DMA channel source address
 *
 * @param[in]  DMACHx :  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  u32Addr:  Source address
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetSourceAddr(DMACHx, u32Addr)        ((DMACHx)->DMACHSA.all = (u32Addr))




/******************************************************************************
 * @brief      Set DMA channel destination address
 *
 * @param[in]  DMACHx :  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  u32Addr:  Destination address
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetDestinationAddr(DMACHx, u32Addr)   ((DMACHx)->DMACHDA.all = (u32Addr))




/******************************************************************************
 * @brief      Set DMA transfer type
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eType :  DMA transfer type
 *                      Please go to DMA_TransferTypeEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetTransferType(DMACHx, eType)        ((DMACHx)->DMACHCTL0.bit.TT = eType)




/******************************************************************************
 * @brief      Set source-end burst transaction length
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eLen  :  Number of data items to be read from the source every time
 *                      when a source burst transaction request is made
 *                      Please go to DMA_BurstTransEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetSourceBurstLen(DMACHx, eLen)       ((DMACHx)->DMACHCTL0.bit.SSIZE = eLen)




/******************************************************************************
 * @brief      Set destination-end burst transaction length
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eLen  :  Number of data items to be written to the destination every time
 *                      when a destination burst transaction request is made
 *                      Please go to DMA_BurstTransEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetDestinationBurstLen(DMACHx, eLen)  ((DMACHx)->DMACHCTL0.bit.DSIZE = eLen)




/******************************************************************************
 * @brief      Set source-end address mode
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eMode :  Address mode
 *                      Please go to DMA_AddressModeEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetSourceAddrMode(DMACHx, eMode)      ((DMACHx)->DMACHCTL0.bit.SINC = eMode)




/******************************************************************************
 * @brief      Set destination-end address mode
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eMode :  Address mode
 *                      Please go to DMA_AddressModeEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetDestinationAddrMode(DMACHx, eMode) ((DMACHx)->DMACHCTL0.bit.DINC = eMode)




/******************************************************************************
 * @brief      Enable DMA channel interrupt
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_EnableChannelInt(DMACHx)        ((DMACHx)->DMACHCTL0.bit.IE = DMACHCTL0_BIT_IE_ENABLE)




/******************************************************************************
 * @brief      Disable DMA channel interrupt
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_DisableChannelInt(DMACHx)       ((DMACHx)->DMACHCTL0.bit.IE = DMACHCTL0_BIT_IE_DISABLE)




/******************************************************************************
 * @brief      Set DMA channel block transfer size
 *
 * @param[in]  DMACHx :  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  u16Size:  Block transfer size (in words)
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetBlockTransferSize(DMACHx, u16Size) ((DMACHx)->DMACHCTL1.bit.BLKTS = u16Size)




/******************************************************************************
 * @brief      Set source-end handshaking interface
 *
 * @param[in]  DMACHx: Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eType : Handshaking type
 *                     Please go to DMA_HandshakeTypeEnum for details
 * @param[in]  ePol  : Handshaking interface polarity
 *                     Please go to DMA_HandshakePolEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
__STATIC_INLINE void DMA_SetSourceHandShake(DMACH_REGS *DMACHx, DMA_HandshakeTypeEnum eType, DMA_HandshakePolEnum ePol)
{
    DMACHx->DMACHCTL2.bit.SSWHS = eType;
    DMACHx->DMACHCTL2.bit.SHSPOL = ePol;
}




/******************************************************************************
 * @brief      Set destination-end handshaking interface
 *
 * @param[in]  DMACHx: Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eType : Handshaking type
 *                     Please go to DMA_HandshakeTypeEnum for details
 * @param[in]  ePol  : Handshaking interface polarity
 *                     Please go to DMA_HandshakePolEnum for details
 *
 * @return     none
 *
 * @example    Only the following handshaking is supported:
 *             DMA_SetDestinationHandShake(DMACH0, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
 *
 ******************************************************************************/
__STATIC_INLINE void DMA_SetDestinationHandShake(DMACH_REGS *DMACHx, DMA_HandshakeTypeEnum eType, DMA_HandshakePolEnum ePol)
{
    DMACHx->DMACHCTL2.bit.DSWHS = eType;
    DMACHx->DMACHCTL2.bit.DHSPOL = ePol;
}




/******************************************************************************
 * @brief      Get the channel FIFO state (empty or not)
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return      0 - FIFO not empty
 *              1 - FIFO empty
 *
 ******************************************************************************/
#define DMA_IsFIFOEmpty(DMACHx)             ((DMACHx)->DMACHCTL2.bit.FIFOEMPTY)




/******************************************************************************
 * @brief      Enable DMA channel Suspend
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_EnableSuspend(DMACHx)           ((DMACHx)->DMACHCTL2.bit.SUSPEND = DMACHCTL2_BIT_SUSPEND_ENABLE)




/******************************************************************************
 * @brief      Disable DMA channel suspend
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_DisableSuspend(DMACHx)          ((DMACHx)->DMACHCTL2.bit.SUSPEND = DMACHCTL2_BIT_SUSPEND_DISABLE)




/******************************************************************************
 * @brief      Set DMA channel priority
 *
 * @param[in]  DMACHx    :  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  u8Priority:  Channel priority
 *                          Please go to DMACHCTL2_BIT_PRIORITY for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetPriority(DMACHx, u8Priority) ((DMACHx)->DMACHCTL2.bit.PRIORITY = u8Priority)




/******************************************************************************
 * @brief      Set detination-end peripheral device
 *
 * @param[in]  DMACHx : Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  ePer   : Peripheral device
 *                      Please go to DMA_PeripheralEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetDestinationPeripheral(DMACHx, ePer)  ((DMACHx)->DMACHCTL3.bit.DPER = ePer)




/******************************************************************************
 * @brief      Set source-end peripheral device
 *
 * @param[in]  DMACHx : Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  ePer   : Peripheral device
 *                      Please go to DMA_PeripheralEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetSourcePeripheral(DMACHx, ePer)       ((DMACHx)->DMACHCTL3.bit.SPER = ePer)




/******************************************************************************
 * @brief      Set FIFO mode for servicing a burst transaction
 *
 * @param[in]  DMACHx  :  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  eFIFOLvl:  FIFO threshold
 *                        Please go to DMACHCTL3_BIT_FIFOTH for details
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_SetFIFOThreshold(DMACHx, eFIFOLvl)    ((DMACHx)->DMACHCTL3.bit.FIFOTH = eFIFOLvl)




/******************************************************************************
 * @brief      Get DMA channel transfer complete raw interrupt flag
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     0 - Channel transfer not complete
 *             1 - Channel transfer has completed
 *
 ******************************************************************************/
#define DMA_GetTransferCompleteIntRawFlag(eDMACH) (((DMAC->DMATCRAWIF.all) & (0x1UL << eDMACH)) >> (eDMACH))




/******************************************************************************
 * @brief      Get DMA channel error raw interrupt flag
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     0 - Channel error not occur
 *             1 - Channel error occurred
 *
 ******************************************************************************/
#define DMA_GetErrorIntRawFlag(eDMACH)      (((DMAC->DMAERRRAWIF.all) & (0x1UL << eDMACH)) >> (eDMACH))




/******************************************************************************
 * @brief      Get channel transfer complete interrupt flag
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     0 - Channel transfer complete interrupt not occur
 *             1 - Channel transfer complete interrupt occurred
 *
 ******************************************************************************/
#define DMA_GetTransferCompleteIntFlag(eDMACH)    (((DMAC->DMATCIF.all) & (0x1UL << eDMACH)) >> (eDMACH))




/******************************************************************************
 * @brief      Get channel transfer error interrupt flag
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     0 - Channel error interrupt not occur
 *             1 - Channel error interrupt occurred
 *
 ******************************************************************************/
#define DMA_GetErrorIntFlag(eDMACH)         (((DMAC->DMAERRIF.all) & (0x1UL << eDMACH)) >> (eDMACH))




/******************************************************************************
 * @brief      Enable channel transfer complete interrupt
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_EnableTransferCompleteInt(eDMACH)     (DMAC->DMATCIE.all = ((0x1UL << eDMACH) | (0x100UL << eDMACH)))




/******************************************************************************
 * @brief      Disable channel transfer complete interrupt
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_DisableTransferCompleteInt(eDMACH)    (DMAC->DMATCIE.all = (0x100UL << eDMACH))




/******************************************************************************
 * @brief      Enable channel error interrupt
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_EnableErrorInt(eDMACH)          (DMAC->DMAERRIE.all = ((0x1UL << eDMACH) | (0x100UL << eDMACH)))




/******************************************************************************
 * @brief      Disable channel error interrupt
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_DisableErrorInt(eDMACH)         (DMAC->DMAERRIE.all = (0x100UL << eDMACH))




/******************************************************************************
 * @brief      Clear channel transfer complete interrupt
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_ClearTransferCompleteInt(eDMACH)  (DMAC->DMATCIC.all = (0x1UL << eDMACH))




/******************************************************************************
 * @brief      Clear channel error interrupt
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_ClearErrorInt(eDMACH)             (DMAC->DMAERRIC.all = (0x1UL << eDMACH))




/******************************************************************************
 * @brief      Get DMA combined error interrupt flag
 *
 * @param[in]  none
 *
 * @return     0 - Channel error interrupt not occur
 *             1 - A channel error interrupt occurred
 *
 ******************************************************************************/
#define DMA_GetErrorGlobalIntFlag()               (DMAC->DMAIF.bit.ERROR)




/******************************************************************************
 * @brief      Get DMA combined transfer complete interrupt flag
 *
 * @param[in]  none
 *
 * @return     0 - Channel transfer complete interrupt not occur
 *             1 - A channel transfer complete interrupt occurred
 *
 ******************************************************************************/
#define DMA_GetTransferCompleteGlobalIntFlag()    (DMAC->DMAIF.bit.TC)




/******************************************************************************
 * @brief      Enable DMA controller
 *
 * @param[in]  none
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_Enable()                        (DMAC->DMAEN.all = DMAEN_ALL_EN_ENABLE)




/******************************************************************************
 * @brief      Disable DMA controller
 *
 * @param[in]  none
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_Disable()                       (DMAC->DMAEN.all = DMAEN_ALL_EN_DISABLE)




/******************************************************************************
 * @brief      Enable DMA channel
 *
 * @param[in]  eDMACH:  Select the DMA channel, defined by DMA_ChannelEnum
 *
 * @return     none
 *
 ******************************************************************************/
#define DMA_EnableChannelTransfer(eDMACH)   (DMAC->DMACHEN.all = ((0x1UL << eDMACH) | (0x100UL << eDMACH)))




/**
 *  @brief Power Public Function Declaration
 */
void DMA_SetHandShakeWithPeripheral(DMACH_REGS *DMACHx, DMA_PeripheralEnum ePeripheral);


#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* DMA_H */


/******************* (C) COPYRIGHT 2021 SPINTROL LIMITED ***** END OF FILE ****/
