/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.6.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING 
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE 
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE 
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spc2168.h"
#include <stdio.h>


uint32_t *pu32Src;
uint32_t *pu32Dst;
uint32_t i;


 /*************************************************************************************************************************
 *
 * @brief       This example use DMA channel 0 to transfer a single block transfer (block transfer length is 13 words) from
 *              UART to memory,after transaction is done, trigger a complete INT, and check the datum. When we put 16bytes 
 *              into UART FIFO(In this demo code, we set UART bus width 8bits, so every UART FIFO entry will only contain 
 *              one byte, so 16bytes means 16 FIFO entries), a DMA burst transfer is triggered, DMA will tansfer 4 items (
 *              32bits per item = 1 UART FIFO entry), after this burst transfer done, UART FIFO data number is under 16 entries
 *              (16-4=12 entries), so another 4bytes need put into UART FIFO to trigger DMA burst transfer(that is to say, 
 *              after you sent 16bytes to UART FIFO, DMA will transfer 4bytes immediately, there will left 12bytes in FIFO,
 *              at this moument, the total data haven't tansfer yet, and the data left in FIFO are not enough to triger a INT
 *              to CPU, in order to triger the INT, we need to sent other 4bytes), after 8items(DMA block transaction) is done,
 *              DMA coressponding channel will disable automatically.
 *
 *              Key point :
 *                     the total datum transfered by DMA is : DMACH0->DMACHCTL1.bit.BLKTS = 8 bytes, after that, DMA will 
 *                     trigger complete INT singal to CPU.
 *
 *************************************************************************************************************************/


int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO44/45 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_44,GPIO44_UART_TXD);
  GPIO_SetPinChannel(GPIO_45,GPIO45_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Enable global INT for DMA */
  NVIC_EnableIRQ(DMAC_IRQn);
  
  pu32Src = (uint32_t *)(&UART->UARTRBR_UARTTHR_UARTDLL.UARTRBR.all);
  pu32Dst = (uint32_t *)0x20002000;
  
  /* Disable UART */
  UART_Disable(UART);

  /* Config FIFO */
  UART->UARTIIR_UARTFCR.UARTFCR.all = UARTFCR_ALL_TRFIFOE_FIFO_ENABLE |      /*                                          */
                                      UARTFCR_ALL_CLRRF_CLEAR_RX_FIFO |      /*                                          */
                                      UARTFCR_ALL_CLRTF_CLEAR_TX_FIFO |      /*                                          */
                                      UARTFCR_ALL_TXTH_TXFIFO_EMPTY |         /* Interrupt/DMA request when FIFO is empty */
                                      UARTFCR_ALL_BUS32_8BIT |                /*                                          */
                                      UARTFCR_ALL_RXTH_RX_FIFO_THRESHOLD_16;  /* 16 byte or more in FIFO causes interrupt */

  /* Enable UART DMA */
  UART_EnableDMAService(UART);
  
  /* Enable UART */
  UART_Enable(UART);
  
  /* Enable DMA */
  DMA_Enable();
  
  /* Source address */
  DMA_SetSourceAddr(DMACH0, (uint32_t)pu32Src);
  /* Destination address */
  DMA_SetDestinationAddr(DMACH0, (uint32_t)pu32Dst);
  
  /* Transfer type */
  DMA_SetTransferType(DMACH0, DMACHCTL0_BIT_TT_PER_TO_MEM);
  
  /* Source address increment */
  DMA_SetSourceAddrMode(DMACH0, DMA_ADDRESS_NO_CHANGE);
  /* Destination address increment */
  DMA_SetDestinationAddrMode(DMACH0, DMA_ADDRESS_MODE_INCREASE);
  
  /* Source burst transfer length - 4 words = UART RXFIFO Threshold */
  DMA_SetSourceBurstLen(DMACH0, DMACHCTL0_BIT_SSIZE_4_WORD);
  /* Set destination burst transfer length as the same as Source */
  DMA_SetDestinationBurstLen(DMACH0, DMACHCTL0_BIT_DSIZE_4_WORD);
  
  /* Block transfer size */
  DMA_SetBlockTransferSize(DMACH0, 8);
  
  /* Set source handshaking select and handshaking interface polarity */
  DMA_SetSourceHandShake(DMACH0, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
  
  /* Source peripheral */
  DMA_SetSourcePeripheral(DMACH0, DMACHCTL3_BIT_SPER_UART_RX);
  
  /* Enable channel 0 interruput */
  DMA_EnableChannelInt(DMACH0);
  /* Enable Channel 0 transfer complete interrupt */
  DMA_EnableTransferCompleteInt(DMA_CH0);
  
  /* Enable Channel 0 */
  DMA_EnableChannelTransfer(DMA_CH0);

  while(1)
  {

  }
}

void DMAC_IRQHandler(void)
{
  /* Disable FIFO, otherwise, printf will not work */
  UART_DisableFIFO(UART);
  
  if(DMA_GetErrorGlobalIntFlag() == 1)
  {
    printf("DMA Channel Error!\n");
  }
  
  if(DMA_GetTransferCompleteGlobalIntFlag() == 1)
  {
    printf("Channel transfer complete!\n");
  }
  
  if(DMA_GetTransferCompleteIntFlag(DMA_CH0) == 1)
  {
    printf("Channel 0 transfer complete!\n");
    
    for(i = 0; i < 16; i++)
    {
      printf("Dst[%d] = 0x%08X\n", i, pu32Dst[i]);
    }
    
    DMA_ClearTransferCompleteInt(DMA_CH0);
  }
}
