/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.6.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

/* Standard includes. */
#include <stdio.h>
#include <string.h>

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"
#include "semphr.h"


/* product specific includes */
#include "spc2168.h"

#define mainDELAY_LOOP_COUNT 50000000

void vTask1( void *pvParameters )
{
  char * str = NULL;
  str = pvPortMalloc(0x400);
  if(str == NULL)
      printf("alloc 1K memory fail\n");
  else
  {
      printf("alloc 1K memory sucess\n");
      vPortFree(str);
  }
  
  while(1)
  {
      printf("Task 1 is running\n");
      vTaskDelay(1000);
  }
}

void vTask2( void *pvParameters )
{
  const char *pcTaskName = "Task 2 is running\n";
  volatile uint32_t ul; /* volatile to ensure ul is not optimized away. */
  /* As per most tasks, this task is implemented in an infinite loop. */
  for( ;; )
  {
      /* Print out the name of this task. */
      printf("%s\n",pcTaskName);
      /* Delay for a period. */
      for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
      {
      /* This loop is just a very crude delay implementation. There is
      nothing to do in here. Later examples will replace this crude
      loop with a proper delay/sleep function. */
      }
  }
}

void ThreadFunctionTest(void)
{
  /* Creat task 1 */
  xTaskCreate(vTask1,"Vtask 1",512,NULL,1,NULL);
  
  /* Creat tsak 2 */
  xTaskCreate(vTask2,"Vtask 2",512,NULL,1,NULL);
}


 /*************************************************************************************************************************
 *
 * @brief      Chip with CACHE and CAU, the max RAM size can be accessed by CPU is 48KB (from 0x1FFFC000 ~ 0x20007FFF), when
 *             chip woth no CACHE, but has CAU, the max RAM size can be accessed by CPU is 64KB (from 0x1FFF8000 ~ 0x20007FFF),
 *             please take note. If need, change the RAM size and address setting in configure target options interface of the
 *             priject. 
 *             
 *             This demo showed how to use the FreeRTOS to creat two thread, which ware scheduled every 1Ms interactively 
 *             (controled by macro configTICK_RATE_HZ in FreeRTOSConfig.h).
 *
 *************************************************************************************************************************/


int main()
{
  /* 
   * Close Global INT first to prevent conflict bwtween Free RTOS and the
   * Original SYS 
   */
  __set_PRIMASK(1);

  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
  * Init the UART
  *
  * 1.Set the GPIO34/35 as UART FUNC
  *
  * 2.Enable the UART CLK
  *
  * 3.Set the rest para
  */
  GPIO_SetPinChannel(GPIO_44,GPIO44_UART_TXD);
  GPIO_SetPinChannel(GPIO_45,GPIO45_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Write 0 to PRIMASK register to allow interrupt */
  __set_PRIMASK(0);

  /* Creat two task to test the mutil-thread function */
  ThreadFunctionTest();

  /* Start the scheduler. */
  vTaskStartScheduler();

  /* If all is well, the scheduler will now be running, and the following line
  will never be reached.  If the following line does execute, then there was
  insufficient FreeRTOS heap memory available for the idle and/or timer tasks
  to be created.  See the memory management section on the FreeRTOS web site
  for more details. */
  for( ;; );
}

void vApplicationMallocFailedHook( void )
{
  /* vApplicationMallocFailedHook() will only be called if
  configUSE_MALLOC_FAILED_HOOK is set to 1 in FreeRTOSConfig.h.  It is a hook
  function that will get called if a call to pvPortMalloc() fails.
  pvPortMalloc() is called internally by the kernel whenever a task, queue,
  timer or semaphore is created.  It is also called by various parts of the
  demo application.  If heap_1.c or heap_2.c are used, then the size of the
  heap available to pvPortMalloc() is defined by configTOTAL_HEAP_SIZE in
  FreeRTOSConfig.h, and the xPortGetFreeHeapSize() API function can be used
  to query the size of free heap space that remains (although it does not
  provide information on how the remaining heap might be fragmented). */
  taskDISABLE_INTERRUPTS();
  for( ;; );
}

void vApplicationIdleHook( void )
{
  /* vApplicationIdleHook() will only be called if configUSE_IDLE_HOOK is set
  to 1 in FreeRTOSConfig.h.  It will be called on each iteration of the idle
  task.  It is essential that code added to this hook function never attempts
  to block in any way (for example, call xQueueReceive() with a block time
  specified, or call vTaskDelay()).  If the application makes use of the
  vTaskDelete() API function (as this demo application does) then it is also
  important that vApplicationIdleHook() is permitted to return to its calling
  function, because it is the responsibility of the idle task to clean up
  memory allocated by the kernel to any task that has since been deleted. */
}

void vApplicationStackOverflowHook( TaskHandle_t pxTask, char *pcTaskName )
{
  ( void ) pcTaskName;
  ( void ) pxTask;

  /* Run time stack overflow checking is performed if
  configCHECK_FOR_STACK_OVERFLOW is defined to 1 or 2.  This hook
  function is called if a stack overflow is detected. */
  taskDISABLE_INTERRUPTS();
  for( ;; );
}

void vApplicationTickHook( void )
{
  /* This function will be called by each tick interrupt if 
  configUSE_TICK_HOOK is set to 1 in FreeRTOSConfig.h.  User code can be
  added here, but the tick hook is called from an interrupt context, so
  code must not attempt to block, and only the interrupt safe FreeRTOS API
  functions can be used (those that end in FromISR()). */
}
